@@ -450,7 +450,7 @@ static int iptables_add_chain(struct connman_iptables *table,
 				sizeof(struct error_target);
 	entry_head = g_try_malloc0(entry_head_size);
 	if (entry_head == NULL)
-		goto err;
+		goto err_head;
 
 	memset(entry_head, 0, entry_head_size);
 
@@ -463,7 +463,7 @@ static int iptables_add_chain(struct connman_iptables *table,
 	strcpy(error->error, name);
 
 	if (iptables_add_entry(table, entry_head, last, -1) < 0)
-		goto err;
+		goto err_head;
 
 	/* tail entry */
 	entry_return_size = sizeof(struct ipt_entry) +
@@ -488,8 +488,9 @@ static int iptables_add_chain(struct connman_iptables *table,
 	return 0;
 
 err:
-	g_free(entry_head);
 	g_free(entry_return);
+err_head:
+	g_free(entry_head);
 
 	return -ENOMEM;
 }
@@ -902,6 +903,7 @@ static void table_cleanup(struct connman_iptables *table)
 		entry = list->data;
 
 		g_free(entry->entry);
+		g_free(entry);
 	}
 
 	g_list_free(table->entries);
@@ -1087,7 +1089,11 @@ static int iptables_command(int argc, char *argv[])
 			if (xt_t->init != NULL)
 				xt_t->init(xt_t->t);
 			iptables_globals.opts =
-				xtables_merge_options(iptables_globals.opts,
+				xtables_merge_options(
+#if XTABLES_VERSION_CODE > 5
+						     iptables_globals.orig_opts,
+#endif
+						     iptables_globals.opts,
 						     xt_t->extra_opts,
 						     &xt_t->option_offset);
 			if (iptables_globals.opts == NULL)
@@ -1111,7 +1117,11 @@ static int iptables_command(int argc, char *argv[])
 				xt_m->init(xt_m->m);
 			if (xt_m != xt_m->next) {
 				iptables_globals.opts =
-				xtables_merge_options(iptables_globals.opts,
+				xtables_merge_options(
+#if XTABLES_VERSION_CODE > 5
+						iptables_globals.orig_opts,
+#endif
+						iptables_globals.opts,
 						xt_m->extra_opts,
 						&xt_m->option_offset);
 				if (iptables_globals.opts == NULL)
@@ -1300,6 +1310,7 @@ int __connman_iptables_commit(const char *table_name)
 
 	err = iptables_replace(table, repl);
 
+	g_free(repl->counters);
 	g_free(repl);
 
 	if (err < 0)
@@ -1332,6 +1343,8 @@ int __connman_iptables_init(void)
 
 void __connman_iptables_cleanup(void)
 {
+	DBG("");
+
 	g_hash_table_destroy(table_hash);
 
 	xtables_free_opts(1);
