@@ -23,11 +23,13 @@
 #include <config.h>
 #endif
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
+#include <sys/signalfd.h>
 #include <getopt.h>
 #include <sys/stat.h>
 #include <net/if.h>
@@ -40,13 +42,123 @@
 
 #include "connman.h"
 
+static struct {
+	connman_bool_t bg_scan;
+} connman_settings  = {
+	.bg_scan = TRUE,
+};
+
+static GKeyFile *load_config(const char *file)
+{
+	GError *err = NULL;
+	GKeyFile *keyfile;
+
+	keyfile = g_key_file_new();
+
+	g_key_file_set_list_separator(keyfile, ',');
+
+	if (!g_key_file_load_from_file(keyfile, file, 0, &err)) {
+		if (err->code != G_FILE_ERROR_NOENT) {
+			connman_error("Parsing %s failed: %s", file,
+								err->message);
+		}
+
+		g_error_free(err);
+		g_key_file_free(keyfile);
+		return NULL;
+	}
+
+	return keyfile;
+}
+
+static void parse_config(GKeyFile *config)
+{
+	GError *error = NULL;
+	gboolean boolean;
+
+	if (config == NULL)
+		return;
+
+	DBG("parsing main.conf");
+
+	boolean = g_key_file_get_boolean(config, "General",
+						"BackgroundScanning", &error);
+	if (error == NULL)
+		connman_settings.bg_scan = boolean;
+
+	g_clear_error(&error);
+}
+
 static GMainLoop *main_loop = NULL;
 
-static void sig_term(int sig)
-{
-	connman_info("Terminating");
+static unsigned int __terminated = 0;
 
-	g_main_loop_quit(main_loop);
+static gboolean signal_handler(GIOChannel *channel, GIOCondition cond,
+							gpointer user_data)
+{
+	struct signalfd_siginfo si;
+	ssize_t result;
+	int fd;
+
+	if (cond & (G_IO_NVAL | G_IO_ERR | G_IO_HUP))
+		return FALSE;
+
+	fd = g_io_channel_unix_get_fd(channel);
+
+	result = read(fd, &si, sizeof(si));
+	if (result != sizeof(si))
+		return FALSE;
+
+	switch (si.ssi_signo) {
+	case SIGINT:
+	case SIGTERM:
+		if (__terminated == 0) {
+			connman_info("Terminating");
+			g_main_loop_quit(main_loop);
+		}
+
+		__terminated = 1;
+		break;
+	}
+
+	return TRUE;
+}
+
+static guint setup_signalfd(void)
+{
+	GIOChannel *channel;
+	guint source;
+	sigset_t mask;
+	int fd;
+
+	sigemptyset(&mask);
+	sigaddset(&mask, SIGINT);
+	sigaddset(&mask, SIGTERM);
+
+	if (sigprocmask(SIG_BLOCK, &mask, NULL) < 0) {
+		perror("Failed to set signal mask");
+		return 0;
+	}
+
+	fd = signalfd(-1, &mask, 0);
+	if (fd < 0) {
+		perror("Failed to create signal descriptor");
+		return 0;
+	}
+
+	channel = g_io_channel_unix_new(fd);
+
+	g_io_channel_set_close_on_unref(channel, TRUE);
+	g_io_channel_set_encoding(channel, NULL, NULL);
+	g_io_channel_set_buffered(channel, FALSE);
+
+	source = g_io_add_watch(channel,
+				G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
+				signal_handler, NULL);
+
+	g_io_channel_unref(channel);
+
+	return source;
 }
 
 static void disconnect_callback(DBusConnection *conn, void *user_data)
@@ -63,6 +175,7 @@ static gchar *option_nodevice = NULL;
 static gchar *option_noplugin = NULL;
 static gchar *option_wifi = NULL;
 static gboolean option_detach = TRUE;
+static gboolean option_dnsproxy = TRUE;
 static gboolean option_compat = FALSE;
 static gboolean option_version = FALSE;
 
@@ -94,8 +207,11 @@ static GOptionEntry options[] = {
 	{ "nodaemon", 'n', G_OPTION_FLAG_REVERSE,
 				G_OPTION_ARG_NONE, &option_detach,
 				"Don't fork daemon to background" },
+	{ "nodnsproxy", 'r', G_OPTION_FLAG_REVERSE,
+				G_OPTION_ARG_NONE, &option_dnsproxy,
+				"Don't enable DNS Proxy" },
 	{ "compat", 'c', 0, G_OPTION_ARG_NONE, &option_compat,
-				"Enable Network Manager compatibility" },
+				"(obsolete)" },
 	{ "version", 'v', 0, G_OPTION_ARG_NONE, &option_version,
 				"Show version information and exit" },
 	{ NULL },
@@ -113,14 +229,22 @@ const char *connman_option_get_string(const char *key)
 	return NULL;
 }
 
+connman_bool_t connman_setting_get_bool(const char *key)
+{
+	if (g_str_equal(key, "BackgroundScanning") == TRUE)
+		return connman_settings.bg_scan;
+
+	return FALSE;
+}
+
 int main(int argc, char *argv[])
 {
 	GOptionContext *context;
 	GError *error = NULL;
 	DBusConnection *conn;
 	DBusError err;
-	struct sigaction sa;
-	mode_t old_umask;
+	GKeyFile *config;
+	guint signal;
 
 #ifdef HAVE_CAPNG
 	/* Drop capabilities */
@@ -175,7 +299,7 @@ int main(int argc, char *argv[])
 			perror("Failed to create statistics directory");
 	}
 
-	old_umask = umask(077);
+	umask(0077);
 
 	main_loop = g_main_loop_new(NULL, FALSE);
 
@@ -186,6 +310,8 @@ int main(int argc, char *argv[])
 	}
 #endif
 
+	signal = setup_signalfd();
+
 	dbus_error_init(&err);
 
 	conn = g_dbus_setup_bus(DBUS_BUS_SYSTEM, CONNMAN_SERVICE, &err);
@@ -200,31 +326,34 @@ int main(int argc, char *argv[])
 
 	g_dbus_set_disconnect_function(conn, disconnect_callback, NULL, NULL);
 
-	if (option_compat == TRUE) {
-		if (g_dbus_request_name(conn, NM_SERVICE, NULL) == FALSE) {
-			fprintf(stderr, "Can't register compat service\n");
-			option_compat = FALSE;
-		}
-	}
-
 	__connman_log_init(option_debug, option_detach);
 
 	__connman_dbus_init(conn);
 
+	config = load_config(CONFIGDIR "/main.conf");
+
+	parse_config(config);
+
 	__connman_storage_init();
-	__connman_element_init(option_device, option_nodevice);
+	__connman_technology_init();
+	__connman_notifier_init();
+	__connman_location_init();
+	__connman_service_init();
+	__connman_provider_init();
+	__connman_network_init();
+	__connman_device_init(option_device, option_nodevice);
 
 	__connman_agent_init();
 	__connman_iptables_init();
 	__connman_tethering_init();
 	__connman_counter_init();
-	__connman_ondemand_init();
-	__connman_manager_init(option_compat);
+	__connman_manager_init();
 	__connman_profile_init();
 	__connman_config_init();
 	__connman_stats_init();
+	__connman_clock_init();
 
-	__connman_resolver_init();
+	__connman_resolver_init(option_dnsproxy);
 	__connman_ipconfig_init();
 	__connman_rtnl_init();
 	__connman_task_init();
@@ -232,27 +361,34 @@ int main(int argc, char *argv[])
 	__connman_detect_init();
 	__connman_session_init();
 	__connman_timeserver_init();
+	__connman_connection_init();
 
 	__connman_plugin_init(option_plugin, option_noplugin);
 
-	__connman_element_start();
+	__connman_storage_init_profile();
+
+	__connman_rtnl_start();
+	__connman_dhcp_init();
+	__connman_wpad_init();
+	__connman_wispr_init();
+	__connman_rfkill_init();
 
 	g_free(option_device);
 	g_free(option_plugin);
 	g_free(option_nodevice);
 	g_free(option_noplugin);
 
-	memset(&sa, 0, sizeof(sa));
-	sa.sa_handler = sig_term;
-	sigaction(SIGINT, &sa, NULL);
-	sigaction(SIGTERM, &sa, NULL);
-
 	g_main_loop_run(main_loop);
 
-	__connman_element_stop();
+	g_source_remove(signal);
 
+	__connman_rfkill_cleanup();
+	__connman_wispr_cleanup();
+	__connman_wpad_cleanup();
+	__connman_dhcp_cleanup();
+	__connman_provider_cleanup();
 	__connman_plugin_cleanup();
-
+	__connman_connection_cleanup();
 	__connman_timeserver_cleanup();
 	__connman_session_cleanup();
 	__connman_detect_cleanup();
@@ -262,17 +398,21 @@ int main(int argc, char *argv[])
 	__connman_ipconfig_cleanup();
 	__connman_resolver_cleanup();
 
+	__connman_clock_cleanup();
 	__connman_stats_cleanup();
 	__connman_config_cleanup();
 	__connman_profile_cleanup();
 	__connman_manager_cleanup();
-	__connman_ondemand_cleanup();
 	__connman_counter_cleanup();
 	__connman_agent_cleanup();
 	__connman_tethering_cleanup();
 	__connman_iptables_cleanup();
-
-	__connman_element_cleanup();
+	__connman_device_cleanup();
+	__connman_network_cleanup();
+	__connman_service_cleanup();
+	__connman_location_cleanup();
+	__connman_notifier_cleanup();
+	__connman_technology_cleanup();
 	__connman_storage_cleanup();
 
 	__connman_dbus_cleanup();
@@ -283,5 +423,8 @@ int main(int argc, char *argv[])
 
 	g_main_loop_unref(main_loop);
 
+	if (config)
+		g_key_file_free(config);
+
 	return 0;
 }
